Index: kernel/utility/formatters/upload_formatter.php
===================================================================
--- kernel/utility/formatters/upload_formatter.php	(revision 14184)
+++ kernel/utility/formatters/upload_formatter.php	(working copy)
@@ -123,8 +123,17 @@
 			if (is_array($value) && $value['error'] === UPLOAD_ERR_OK) {
 				$max_filesize = isset($options['max_size']) ? $options['max_size'] : MAX_UPLOAD_SIZE;
 
-				if (getArrayValue($options, 'allowed_types') && !in_array($value['type'], $options['allowed_types'])) {
+				if ( getArrayValue($options, 'file_types') && !$this->extensionMatch($value['name'], $options['file_types']) ) {
+					// match by file extensions
 					$error_params = Array (
+						'file_name' => $value['name'],
+						'file_types' => $options['file_types'],
+					);
+					$object->SetError($field_name, 'bad_file_format', 'la_error_InvalidFileFormat', $error_params);
+				}
+				elseif (getArrayValue($options, 'allowed_types') && !in_array($value['type'], $options['allowed_types'])) {
+					// match by mime type provided by web-browser
+					$error_params = Array (
 						'file_type' => $value['type'],
 						'allowed_types' => $options['allowed_types'],
 					);
@@ -175,6 +184,18 @@
 		return $ret;
 	}
 
+	function extensionMatch($filename, $file_types)
+	{
+		if ( preg_match_all('/\*\.(.*?)(;|$)/', $file_types, $regs) ) {
+			$file_extension = mb_strtolower( pathinfo($filename, PATHINFO_EXTENSION) );
+			$file_extensions = array_map('mb_strtolower', $regs[1]);
+
+			return in_array($file_extension, $file_extensions);
+		}
+
+		return true;
+	}
+
 	function getSingleFormat($format)
 	{
 		$single_mapping = Array (
